@@ -4,7 +4,7 @@
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- *
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,6 +13,7 @@ package org.eclipse.jdt.core.tests.compiler.regression;
 import java.io.File;
 import java.util.Map;
 
+import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.core.ToolFactory;
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.core.util.IClassFileAttribute;
@@ -33,7 +34,7 @@ public class VarargsTest extends AbstractComparableTest {
 	// All specified tests which does not belong to the class are skipped...
 	static {
 //		TESTS_NAMES = new String[] { "test000" };
-//		TESTS_NUMBERS = new int[] { 62 };
+//		TESTS_NUMBERS = new int[] { 61 };
 //		TESTS_RANGE = new int[] { 11, -1 };
 	}
 	public static Test suite() {
@@ -2144,6 +2145,27 @@ public class VarargsTest extends AbstractComparableTest {
 	}
 	//https://bugs.eclipse.org/bugs/show_bug.cgi?id=223427
 	public void test061() {
+		String expectedOutput = 
+				"----------\n" +
+				"1. WARNING in X.java (at line 5)\n" +
+				"	Collections.addAll(constantClassSet, String.class, Object.class);\n" +
+				"	^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" +
+				"Type safety: A generic array of Class<? extends Object> is created for a varargs parameter\n" +
+				"----------\n" +
+				"2. ERROR in X.java (at line 6)\n" +
+				"	Zork z;\n" +
+				"	^^^^\n" +
+				"Zork cannot be resolved to a type\n" +
+				"----------\n";
+		if (this.complianceLevel >= ClassFileConstants.JDK1_7) {
+			expectedOutput = 
+					"----------\n" +
+					"1. ERROR in X.java (at line 6)\n" +
+					"	Zork z;\n" +
+					"	^^^^\n" +
+					"Zork cannot be resolved to a type\n" +
+					"----------\n";
+		}
 		this.runNegativeTest(
 				new String[] {
 					"X.java",
@@ -2156,17 +2178,7 @@ public class VarargsTest extends AbstractComparableTest {
 					"	}\n" +
 					"}\n", // =================
 				},
-				"----------\n" +
-				"1. WARNING in X.java (at line 5)\n" +
-				"	Collections.addAll(constantClassSet, String.class, Object.class);\n" +
-				"	^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" +
-				"Type safety : A generic array of Class<? extends Object> is created for a varargs parameter\n" +
-				"----------\n" +
-				"2. ERROR in X.java (at line 6)\n" +
-				"	Zork z;\n" +
-				"	^^^^\n" +
-				"Zork cannot be resolved to a type\n" +
-				"----------\n");
+				expectedOutput);
 	}
 	//https://bugs.eclipse.org/bugs/show_bug.cgi?id=328247
 	public void test062() throws Exception {
@@ -2196,8 +2208,137 @@ public class VarargsTest extends AbstractComparableTest {
 			"  X$1(X arg0, java.lang.Integer $anonymous0, java.lang.String... $anonymous1, java.lang.Float arg3);\n";
 		checkDisassembledClassFile(OUTPUT_DIR + File.separator + "X$1.class", "X$1", expectedOutput);
 	}
+	//safe varargs support
+	public void test063() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+				"Y.java",
+				"import java.util.ArrayList;\n" +
+				"import java.util.List;\n" +
+				"public class Y {\r\n" +
+				"	@SafeVarargs\n" +
+				"	public static <T> List<T> asList(T... a) {\n" + 
+				"		return null;\n" + 
+				"	}\n" +
+				"}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runConformTest(
+			new String[] {
+				"X.java",
+				"import java.util.ArrayList;\n" +
+				"import java.util.List;\n" +
+				"public class X {\r\n" +
+				"	public void bar() {\n" +
+				"		List<? extends Class<?>> classes = Y.asList(String.class, Boolean.class);\n" +
+				"	}\n" +
+				"}",
+			},
+			"",
+			null,
+			false,
+			null,
+			options,
+			null);
+	}
+	public void test064() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runConformTest(
+			new String[] {
+				"X.java",
+				"import java.util.ArrayList;\n" +
+				"import java.util.List;\n" +
+				"public class X {\r\n" +
+				"	@SafeVarargs\n" +
+				"	public static <T> List<T> asList(T... a) {\n" + 
+				"		return null;\n" + 
+				"	}\n" +
+				"	public void bar() {\n" +
+				"		List<? extends Class<?>> classes = X.asList(String.class, Boolean.class);\n" +
+				"	}\n" +
+				"}",
+			},
+			"",
+			null,
+			false,
+			null,
+			options,
+			null);
+	}
+	public void test065() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runConformTest(
+			new String[] {
+				"X.java",
+				"import java.util.ArrayList;\n" +
+				"import java.util.List;\n" +
+				"public class X {\r\n" +
+				"	@SafeVarargs\n" +
+				"	public static <T> List<T> asList(T... a) {\n" + 
+				"		return null;\n" + 
+				"	}\n" +
+				"	public void bar() {\n" +
+				"		List<List<String>> classes = X.asList(X.asList(\"Hello\", \"World\"));\n" +
+				"	}\n" +
+				"}",
+			},
+			"",
+			null,
+			false,
+			null,
+			options,
+			null);
+	}
 	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337093
-	public void test063() {
+	public void test066() {
 		Map options = getCompilerOptions();
 		options.put(CompilerOptions.OPTION_ReportMissingOverrideAnnotationForInterfaceMethodImplementation, CompilerOptions.DISABLED);
 		this.runNegativeTest(
@@ -2224,14 +2365,395 @@ public class VarargsTest extends AbstractComparableTest {
 					"    }\n" +
 					"}\n", // =================
 				},
+				this.complianceLevel < ClassFileConstants.JDK1_7 ?
 				"----------\n" + 
 				"1. WARNING in X.java (at line 18)\n" + 
 				"	new IteratorChain<Number>(null, null);\n" + 
 				"	^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" + 
-				"Type safety : A generic array of Collection<? extends Number> is created for a varargs parameter\n" + 
+				"Type safety: A generic array of Collection<? extends Number> is created for a varargs parameter\n" + 
+				"----------\n":
+				"----------\n" + 
+				"1. WARNING in X.java (at line 5)\n" + 
+				"	public IteratorChain(Collection<? extends T> a, Collection<? extends T> b, Collection<? extends T> ... collections) {\n" + 
+				"	                                                                                                       ^^^^^^^^^^^\n" + 
+				"Type safety: Potential heap pollution via varargs parameter collections\n" + 
+				"----------\n" + 
+				"2. WARNING in X.java (at line 18)\n" + 
+				"	new IteratorChain<Number>(null, null);\n" + 
+				"	^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" + 
+				"Type safety: A generic array of Collection<? extends Number> is created for a varargs parameter\n" + 
 				"----------\n",
 				null, 
 				true,
 				options);		
 	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337799
+	public void test067() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"	@SafeVarargs\n" +
+				"	public static <T> List<T> asList() {  // Error, not varargs\n" +
+				"		return null;\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public <T> List<T> asList2(T ... a) {    // error not static or final\n" +
+				"		return null;\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public static <T> List<T> asList3(T ... a) {  // OK, varargs & static\n" +
+				"		return null;\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public final <T> List<T> asList4(T ... a) {  // OK varargs & final\n" +
+				"		return null;\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public final static <T> List<T> asList5(T ... a) {  // OK, varargs & static & final\n" +
+				"		return null;\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public int b;\n" +
+				"}\n" +
+				"interface I {\n" +
+				"	@SafeVarargs\n" +
+				"	public  <T> List<T> asList(T ... t);\n" +
+				"}\n",
+			},
+			"----------\n" + 
+			"1. ERROR in X.java (at line 4)\n" + 
+			"	public static <T> List<T> asList() {  // Error, not varargs\n" + 
+			"	                          ^^^^^^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to fixed arity method asList\n" + 
+			"----------\n" + 
+			"2. ERROR in X.java (at line 8)\n" + 
+			"	public <T> List<T> asList2(T ... a) {    // error not static or final\n" + 
+			"	                   ^^^^^^^^^^^^^^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to non-final instance method asList2\n" + 
+			"----------\n" + 
+			"3. ERROR in X.java (at line 23)\n" + 
+			"	@SafeVarargs\n" + 
+			"	^^^^^^^^^^^^\n" + 
+			"The annotation @SafeVarargs is disallowed for this location\n" + 
+			"----------\n" + 
+			"4. ERROR in X.java (at line 28)\n" + 
+			"	public  <T> List<T> asList(T ... t);\n" + 
+			"	                    ^^^^^^^^^^^^^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to non-final instance method asList\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337799
+	public void test067b() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"	@SafeVarargs\n" +
+				"	public X() {  // Error, not varargs\n" +
+				"	}\n" +
+				"	@SafeVarargs\n" +
+				"	public <T> X(T ... a) {\n" +
+				"	}\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. ERROR in X.java (at line 4)\n" + 
+			"	public X() {  // Error, not varargs\n" + 
+			"	       ^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to fixed arity method X\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337795 (make sure there is no warning if vararg parameter is reifiable)
+	public void test068() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"	public <T> X(String ... a) {\n" +
+				"	}\n" +
+				"	public <T> X(int i, String ... a) {\n" +
+				"	}\n" +
+				"   public <T> List<T> asList(String ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"   public <T> List<T> asList(Zork t, String ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. ERROR in X.java (at line 10)\n" + 
+			"	public <T> List<T> asList(Zork t, String ... a) {\n" + 
+			"	                          ^^^^\n" + 
+			"Zork cannot be resolved to a type\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337795 (make sure there is a warning if vararg parameter is not reifiable)
+	public void test068b() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"	public <T> X(T ... a) {\n" +
+				"	}\n" +
+				"	public <T> X(int i, T ... a) {\n" +
+				"	}\n" +
+				"   public <T> List<T> asList(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"   public <T> List<T> asList(T t, T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. WARNING in X.java (at line 3)\n" + 
+			"	public <T> X(T ... a) {\n" + 
+			"	                   ^\n" + 
+			"Type safety: Potential heap pollution via varargs parameter a\n" + 
+			"----------\n" + 
+			"2. WARNING in X.java (at line 5)\n" + 
+			"	public <T> X(int i, T ... a) {\n" + 
+			"	                          ^\n" + 
+			"Type safety: Potential heap pollution via varargs parameter a\n" + 
+			"----------\n" + 
+			"3. WARNING in X.java (at line 7)\n" + 
+			"	public <T> List<T> asList(T ... a) {\n" + 
+			"	                                ^\n" + 
+			"Type safety: Potential heap pollution via varargs parameter a\n" + 
+			"----------\n" + 
+			"4. WARNING in X.java (at line 10)\n" + 
+			"	public <T> List<T> asList(T t, T ... a) {\n" + 
+			"	                                     ^\n" + 
+			"Type safety: Potential heap pollution via varargs parameter a\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337795
+	public void test068c() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"   @SafeVarargs\n" +
+				"	public <T> X(T ... a) {\n" +
+				"	}\n" +
+				"   @SafeVarargs\n" +
+				"	public <T> X(int i, T ... a) {\n" +
+				"	}\n" +
+				"   @SafeVarargs\n" +
+				"   public <T> List<T> asList(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"   @SafeVarargs\n" +
+				"   public <T> List<T> asList(T t, T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. ERROR in X.java (at line 10)\n" + 
+			"	public <T> List<T> asList(T ... a) {\n" + 
+			"	                   ^^^^^^^^^^^^^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to non-final instance method asList\n" + 
+			"----------\n" + 
+			"2. ERROR in X.java (at line 14)\n" + 
+			"	public <T> List<T> asList(T t, T ... a) {\n" + 
+			"	                   ^^^^^^^^^^^^^^^^^^^^\n" + 
+			"@SafeVarargs annotation cannot be applied to non-final instance method asList\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337795
+	public void test068d() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"   @SafeVarargs\n" +
+				"   public static <T> List<T> asList(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"   public static <T> List<T> asList2(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"	List<? extends Class<?>> classes; \n" +
+				"   {\n" +
+				"     classes = X.asList(String.class, Boolean.class);\n" +
+				"	  classes = X.asList2(String.class, Boolean.class);\n" +
+				"   }\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. WARNING in X.java (at line 7)\n" + 
+			"	public static <T> List<T> asList2(T ... a) {\n" + 
+			"	                                        ^\n" + 
+			"Type safety: Potential heap pollution via varargs parameter a\n" + 
+			"----------\n" + 
+			"2. WARNING in X.java (at line 13)\n" + 
+			"	classes = X.asList2(String.class, Boolean.class);\n" + 
+			"	          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" + 
+			"Type safety: A generic array of Class<? extends Object&Serializable&Comparable<?>> is created for a varargs parameter\n" + 
+			"----------\n");
+	}
+	// https://bugs.eclipse.org/bugs/show_bug.cgi?id=337795 (test effect of SuppressWarnings (should suppress at declaration site, but not at call site)
+	public void test068e() throws Exception {
+		if (this.complianceLevel < ClassFileConstants.JDK1_7) return;
+		this.runConformTest(
+			new String[] {
+				"java/lang/SafeVarargs.java",
+				"package java.lang;\n" +
+				"import java.lang.annotation.Retention;\n" + 
+				"import java.lang.annotation.Target;\n" + 
+				"import static java.lang.annotation.RetentionPolicy.RUNTIME;\n" + 
+				"import static java.lang.annotation.ElementType.CONSTRUCTOR;\n" + 
+				"import static java.lang.annotation.ElementType.METHOD;\n" + 
+				"\n" + 
+				"@Retention(value=RUNTIME)\n" + 
+				"@Target(value={CONSTRUCTOR,METHOD})\n" + 
+				"public @interface SafeVarargs {}",
+			},
+			"");
+		Map options = getCompilerOptions();
+		options.put(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION, JavaCore.ERROR);
+		this.runNegativeTest(
+			new String[] {
+				"X.java",
+				"import java.util.List;\n" +
+				"public class X {\n" +
+				"   @SafeVarargs\n" +
+				"   public static <T> List<T> asList(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"   @SuppressWarnings(\"unchecked\")\n" +
+				"   public static <T> List<T> asList2(T ... a) {\n" +
+				"       return null;\n" +
+				"   }\n" +
+				"	List<? extends Class<?>> classes; \n" +
+				"   {\n" +
+				"     classes = X.asList(String.class, Boolean.class);\n" +
+				"	  classes = X.asList2(String.class, Boolean.class);\n" +
+				"   }\n" +
+				"}\n"
+			},
+			"----------\n" + 
+			"1. WARNING in X.java (at line 14)\n" + 
+			"	classes = X.asList2(String.class, Boolean.class);\n" + 
+			"	          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n" + 
+			"Type safety: A generic array of Class<? extends Object&Serializable&Comparable<?>> is created for a varargs parameter\n" + 
+			"----------\n");
+	}
 }
