@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -337,6 +337,9 @@ public class ASTMatcherTest extends org.eclipse.jdt.core.tests.junit.extension.T
 		public boolean match(ContinueStatement node, Object other) {
 			return standardBody(node, other, this.superMatch ? super.match(node, other) : false);
 		}
+		public boolean match(UnionType node, Object other) {
+			return standardBody(node, other, this.superMatch ? super.match(node, other) : false);
+		}
 		public boolean match(DoStatement node, Object other) {
 			return standardBody(node, other, this.superMatch ? super.match(node, other) : false);
 		}
@@ -648,6 +651,15 @@ public class ASTMatcherTest extends org.eclipse.jdt.core.tests.junit.extension.T
 		basicMatch(x1);
 	}
 
+	public void testUnionType() {
+		if (this.ast.apiLevel() < AST.JLS4) {
+			return;
+		}
+		UnionType x1 = this.ast.newUnionType();
+		x1.types().add(this.ast.newSimpleType(this.N1));
+		basicMatch(x1);
+	}
+
 	public void testArrayType() {
 		Type x0 = this.ast.newPrimitiveType(PrimitiveType.CHAR);
 		Type x1 = this.ast.newArrayType(x0);
@@ -1142,6 +1154,27 @@ public class ASTMatcherTest extends org.eclipse.jdt.core.tests.junit.extension.T
 		x1.setFinally(this.ast.newBlock());
 		basicMatch(x1);
 	}
+	public void testTryStatementWithResources() {
+		if (this.ast.apiLevel() < AST.JLS4) {
+			return;
+		}
+		TryStatement x1 = this.ast.newTryStatement();
+		VariableDeclarationExpression var = this.ast.newVariableDeclarationExpression(this.W1);
+		var.setType(this.T1);
+		var.fragments().add(this.W2);
+		x1.resources().add(var);
+		x1.setBody(this.B1);
+		CatchClause c1 = this.ast.newCatchClause();
+		c1.setException(this.V1);
+		c1.setBody(this.ast.newBlock());
+		x1.catchClauses().add(c1);
+		CatchClause c2 = this.ast.newCatchClause();
+		c2.setException(this.V2);
+		c2.setBody(this.ast.newBlock());
+		x1.catchClauses().add(c2);
+		x1.setFinally(this.ast.newBlock());
+		basicMatch(x1);
+	}
 	/** @deprecated using deprecated code */
 	public void testTypeDeclaration() {
 		TypeDeclaration x1 = this.ast.newTypeDeclaration();
