@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2010 IBM Corporation and others.
+ * Copyright (c) 2004, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- *
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -597,4 +597,12 @@ public class CompletionTestsRequestor2 extends CompletionRequestor {
 	public void setComputeEnclosingElement(boolean computeEnclosingElement) {
 		this.computeEnclosingElement = computeEnclosingElement;
 	}
+	
+	public boolean canUseDiamond(int proposalNo) {
+		if (proposalNo < this.proposals.length && this.proposals[proposalNo] != null) {
+			System.out.println(this.proposals[proposalNo]);
+			return this.proposals[proposalNo].canUseDiamond(this.context);
+		}
+		return false;
+	}
 }
