@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2008 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -16,16 +16,18 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import org.eclipse.core.expressions.Expression;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.StyledText;
-import org.eclipse.swt.custom.VerifyKeyListener;
+import org.eclipse.swt.events.DisposeEvent;
+import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.events.FocusEvent;
 import org.eclipse.swt.events.FocusListener;
 import org.eclipse.swt.events.ModifyEvent;
 import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.events.SelectionListener;
-import org.eclipse.swt.events.VerifyEvent;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
@@ -46,13 +48,19 @@ import org.eclipse.jface.action.IMenuListener;
 import org.eclipse.jface.action.IMenuManager;
 import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.commands.ActionHandler;
+import org.eclipse.jface.dialogs.IDialogConstants;
+import org.eclipse.jface.dialogs.IDialogSettings;
+import org.eclipse.jface.dialogs.StatusDialog;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 
+import org.eclipse.jface.text.BadLocationException;
 import org.eclipse.jface.text.Document;
 import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.ITextListener;
 import org.eclipse.jface.text.ITextOperationTarget;
 import org.eclipse.jface.text.ITextViewer;
@@ -64,25 +72,31 @@ import org.eclipse.jface.text.templates.Template;
 import org.eclipse.jface.text.templates.TemplateContextType;
 import org.eclipse.jface.text.templates.TemplateException;
 
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.ActiveShellExpression;
+import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.handlers.IHandlerService;
 import org.eclipse.ui.texteditor.ITextEditorActionConstants;
+import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
 import org.eclipse.ui.texteditor.IUpdate;
+import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;
 
 import org.eclipse.jdt.ui.IContextMenuConstants;
 import org.eclipse.jdt.ui.PreferenceConstants;
+import org.eclipse.jdt.ui.text.IJavaPartitions;
 import org.eclipse.jdt.ui.text.JavaTextTools;
 
 import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
 import org.eclipse.jdt.internal.ui.JavaPlugin;
-import org.eclipse.jdt.internal.ui.dialogs.StatusDialog;
 import org.eclipse.jdt.internal.ui.dialogs.StatusInfo;
 import org.eclipse.jdt.internal.ui.javaeditor.JavaSourceViewer;
-import org.eclipse.jdt.internal.ui.text.IJavaPartitions;
 import org.eclipse.jdt.internal.ui.text.template.preferences.TemplateVariableProcessor;
-import org.eclipse.jdt.internal.ui.util.SWTUtil;
 
 /**
  * Dialog to edit a template.
+ * <p>
+ * <strong>Note:</strong> This is a copy of org.eclipse.ui.texteditor.templates.TemplatePreferencePage.EditTemplateDialog
+ * which we should try to eliminate (see: https://bugs.eclipse.org/bugs/show_bug.cgi?id=208865).
+ * </p>
  */
 public class EditTemplateDialog extends StatusDialog {
 
@@ -91,7 +105,7 @@ public class EditTemplateDialog extends StatusDialog {
 		private int fOperationCode= -1;
 		private ITextOperationTarget fOperationTarget;
 	
-		/** 
+		/**
 		 * Creates a new action.
 		 * 
 		 * @param viewer the viewer
@@ -110,7 +124,10 @@ public class EditTemplateDialog extends StatusDialog {
 		 * @see Action#firePropertyChange(String, Object, Object)
 		 */
 		public void update() {
-	
+			// XXX: workaround for https://bugs.eclipse.org/bugs/show_bug.cgi?id=206111
+			if (fOperationCode == ITextOperationTarget.REDO)
+				return;
+
 			boolean wasEnabled= isEnabled();
 			boolean isEnabled= (fOperationTarget != null && fOperationTarget.canDoOperation(fOperationCode));
 			setEnabled(isEnabled);
@@ -128,24 +145,25 @@ public class EditTemplateDialog extends StatusDialog {
 				fOperationTarget.doOperation(fOperationCode);
 			}
 		}
-	}	
+	}
 
-	private final Template fTemplate;
+	private Template fTemplate;
 	
 	private Text fNameText;
 	private Text fDescriptionText;
 	private Combo fContextCombo;
-	private SourceViewer fPatternEditor;	
+	private SourceViewer fPatternEditor;
 	private Button fInsertVariableButton;
+	private Button fAutoInsertCheckbox;
 	private boolean fIsNameModifiable;
 
 	private StatusInfo fValidationStatus;
-	private boolean fSuppressError= true; // #4354	
+	private boolean fSuppressError= true; // https://bugs.eclipse.org/bugs/show_bug.cgi?id=4354
 	private Map fGlobalActions= new HashMap(10);
-	private List fSelectionActions = new ArrayList(3);	
+	private List fSelectionActions = new ArrayList(3);
 	private String[][] fContextTypes;
 	
-	private ContextTypeRegistry fContextTypeRegistry; 
+	private ContextTypeRegistry fContextTypeRegistry;
 	
 	private final TemplateVariableProcessor fTemplateProcessor= new TemplateVariableProcessor();
 		
@@ -161,26 +179,23 @@ public class EditTemplateDialog extends StatusDialog {
 	public EditTemplateDialog(Shell parent, Template template, boolean edit, boolean isNameModifiable, ContextTypeRegistry registry) {
 		super(parent);
 		
-		setShellStyle(getShellStyle() | SWT.MAX | SWT.RESIZE);
-		
 		String title= edit
-			? PreferencesMessages.getString("EditTemplateDialog.title.edit") //$NON-NLS-1$
-			: PreferencesMessages.getString("EditTemplateDialog.title.new"); //$NON-NLS-1$
+			? PreferencesMessages.EditTemplateDialog_title_edit
+			: PreferencesMessages.EditTemplateDialog_title_new;
 		setTitle(title);
 
 		fTemplate= template;
 		fIsNameModifiable= isNameModifiable;
 		
-		// XXX workaround for bug 63313 - disabling prefix until fixed.
-//		String delim= new Document().getLegalLineDelimiters()[0];
+		String delim= new Document().getLegalLineDelimiters()[0];
 		
 		List contexts= new ArrayList();
 		for (Iterator it= registry.contextTypes(); it.hasNext();) {
 			TemplateContextType type= (TemplateContextType) it.next();
-//			if (type.getId().equals("javadoc")) //$NON-NLS-1$
-//				contexts.add(new String[] { type.getId(), type.getName(), "/**" + delim }); //$NON-NLS-1$
-//			else
-				contexts.add(new String[] { type.getId(), type.getName(), "" }); //$NON-NLS-1$
+			if (type.getId().equals("javadoc")) //$NON-NLS-1$
+				contexts.add(new String[] { type.getId(), type.getName(), "/**" + delim }); //$NON-NLS-1$
+			else
+				contexts.add(0, new String[] { type.getId(), type.getName(), "" }); //$NON-NLS-1$
 		}
 		fContextTypes= (String[][]) contexts.toArray(new String[contexts.size()][]);
 				
@@ -191,19 +206,22 @@ public class EditTemplateDialog extends StatusDialog {
 		TemplateContextType type= fContextTypeRegistry.getContextType(template.getContextTypeId());
 		fTemplateProcessor.setContextType(type);
 	}
+
+	/*
+	 * @see org.eclipse.jface.dialogs.Dialog#isResizable()
+	 * @since 3.4
+	 */
+	protected boolean isResizable() {
+		return true;
+	}
 	
 	/*
 	 * @see org.eclipse.jdt.internal.ui.dialogs.StatusDialog#create()
 	 */
 	public void create() {
 		super.create();
-		// update initial ok button to be disabled for new templates 
-		boolean valid= fNameText == null || fNameText.getText().trim().length() != 0;
-		if (!valid) {
-			StatusInfo status = new StatusInfo();
-			status.setError(PreferencesMessages.getString("EditTemplateDialog.error.noname")); //$NON-NLS-1$
-			updateButtonsEnableState(status);
- 		}
+		updateStatusAndButtons();
+		getButton(IDialogConstants.OK_ID).setEnabled(getStatus().isOK());
 	}
 	
 	/*
@@ -213,6 +231,11 @@ public class EditTemplateDialog extends StatusDialog {
 		Composite parent= new Composite(ancestor, SWT.NONE);
 		GridLayout layout= new GridLayout();
 		layout.numColumns= 2;
+		layout.marginHeight= convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
+		layout.marginWidth= convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
+		layout.verticalSpacing= convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
+		layout.horizontalSpacing= convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
+
 		parent.setLayout(layout);
 		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
 		
@@ -223,12 +246,12 @@ public class EditTemplateDialog extends StatusDialog {
 		};
 		
 		if (fIsNameModifiable) {
-			createLabel(parent, PreferencesMessages.getString("EditTemplateDialog.name")); //$NON-NLS-1$	
+			createLabel(parent, PreferencesMessages.EditTemplateDialog_name);
 			
 			Composite composite= new Composite(parent, SWT.NONE);
 			composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
-			layout= new GridLayout();		
-			layout.numColumns= 3;
+			layout= new GridLayout();
+			layout.numColumns= 4;
 			layout.marginWidth= 0;
 			layout.marginHeight= 0;
 			composite.setLayout(layout);
@@ -242,12 +265,12 @@ public class EditTemplateDialog extends StatusDialog {
 				public void focusLost(FocusEvent e) {
 					if (fSuppressError) {
 						fSuppressError= false;
-						updateButtons();
+						updateStatusAndButtons();
 					}
 				}
 			});
 			
-			createLabel(composite, PreferencesMessages.getString("EditTemplateDialog.context")); //$NON-NLS-1$		
+			createLabel(composite, PreferencesMessages.EditTemplateDialog_context);
 			fContextCombo= new Combo(composite, SWT.READ_ONLY);
 	
 			for (int i= 0; i < fContextTypes.length; i++) {
@@ -255,37 +278,40 @@ public class EditTemplateDialog extends StatusDialog {
 			}
 	
 			fContextCombo.addModifyListener(listener);
+			
+			fAutoInsertCheckbox= createCheckbox(composite, PreferencesMessages.EditTemplateDialog_autoinsert);
+			fAutoInsertCheckbox.setSelection(fTemplate.isAutoInsertable());
 		}
 		
-		createLabel(parent, PreferencesMessages.getString("EditTemplateDialog.description")); //$NON-NLS-1$		
+		createLabel(parent, PreferencesMessages.EditTemplateDialog_description);
 		
 		int descFlags= fIsNameModifiable ? SWT.BORDER : SWT.BORDER | SWT.READ_ONLY;
 		fDescriptionText= new Text(parent, descFlags );
-		fDescriptionText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));	
+		fDescriptionText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		
 		fDescriptionText.addModifyListener(listener);
 
-		Label patternLabel= createLabel(parent, PreferencesMessages.getString("EditTemplateDialog.pattern")); //$NON-NLS-1$
+		Label patternLabel= createLabel(parent, PreferencesMessages.EditTemplateDialog_pattern);
 		patternLabel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
 		fPatternEditor= createEditor(parent);
 		
-		Label filler= new Label(parent, SWT.NONE);		
+		Label filler= new Label(parent, SWT.NONE);
 		filler.setLayoutData(new GridData());
 		
 		Composite composite= new Composite(parent, SWT.NONE);
-		layout= new GridLayout();		
+		layout= new GridLayout();
 		layout.marginWidth= 0;
 		layout.marginHeight= 0;
-		composite.setLayout(layout);		
+		composite.setLayout(layout);
 		composite.setLayoutData(new GridData());
 		
 		fInsertVariableButton= new Button(composite, SWT.NONE);
-		fInsertVariableButton.setLayoutData(getButtonGridData(fInsertVariableButton));
-		fInsertVariableButton.setText(PreferencesMessages.getString("EditTemplateDialog.insert.variable")); //$NON-NLS-1$
+		fInsertVariableButton.setLayoutData(getButtonGridData());
+		fInsertVariableButton.setText(PreferencesMessages.EditTemplateDialog_insert_variable);
 		fInsertVariableButton.addSelectionListener(new SelectionListener() {
 			public void widgetSelected(SelectionEvent e) {
 				fPatternEditor.getTextWidget().setFocus();
-				fPatternEditor.doOperation(ISourceViewer.CONTENTASSIST_PROPOSALS);			
+				fPatternEditor.doOperation(ISourceViewer.CONTENTASSIST_PROPOSALS);
 			}
 
 			public void widgetDefaultSelected(SelectionEvent e) {}
@@ -308,38 +334,37 @@ public class EditTemplateDialog extends StatusDialog {
 	protected void doTextWidgetChanged(Widget w) {
 		if (w == fNameText) {
 			fSuppressError= false;
-			String name= fNameText.getText();
-			fTemplate.setName(name);
-			updateButtons();			
+			updateStatusAndButtons();
 		} else if (w == fContextCombo) {
-			String name= fContextCombo.getText();
-			String contextId= getContextId(name);
-			fTemplate.setContextTypeId(contextId);
+			String contextId= getContextId();
 			fTemplateProcessor.setContextType(fContextTypeRegistry.getContextType(contextId));
+			IDocument document= fPatternEditor.getDocument();
+			String prefix= getPrefix();
+			document.set(prefix + getPattern());
+			fPatternEditor.setVisibleRegion(prefix.length(), document.getLength() - prefix.length());
+			updateStatusAndButtons();
 		} else if (w == fDescriptionText) {
-			String desc= fDescriptionText.getText();
-			fTemplate.setDescription(desc);
-		}	
+			// nothing
+		}
 	}
 	
-	private String getContextId(String name) {
-		if (name == null)
-			return name;
-		
-		for (int i= 0; i < fContextTypes.length; i++) {
-			if (name.equals(fContextTypes[i][1])) {
-				return fContextTypes[i][0];	
+	private String getContextId() {
+		if (fContextCombo != null && !fContextCombo.isDisposed()) {
+			String name= fContextCombo.getText();
+			for (int i= 0; i < fContextTypes.length; i++) {
+				if (name.equals(fContextTypes[i][1])) {
+					return fContextTypes[i][0];
+				}
 			}
 		}
-		return name;
+		
+		return fTemplate.getContextTypeId();
 	}
 
 	protected void doSourceChanged(IDocument document) {
 		String text= document.get();
-		String prefix= getPrefix();
-		fTemplate.setPattern(text.substring(prefix.length(), text.length()));
 		fValidationStatus.setOK();
-		TemplateContextType contextType= fContextTypeRegistry.getContextType(fTemplate.getContextTypeId());
+		TemplateContextType contextType= fContextTypeRegistry.getContextType(getContextId());
 		if (contextType != null) {
 			try {
 				contextType.validate(text);
@@ -348,14 +373,12 @@ public class EditTemplateDialog extends StatusDialog {
 			}
 		}
 
-		updateUndoAction();
-		updateButtons();
-	}	
+		updateAction(ITextEditorActionConstants.UNDO);
+		updateStatusAndButtons();
+	}
 
-	private static GridData getButtonGridData(Button button) {
+	private static GridData getButtonGridData() {
 		GridData data= new GridData(GridData.FILL_HORIZONTAL);
-		data.heightHint= SWTUtil.getButtonHeightHint(button);
-	
 		return data;
 	}
 
@@ -367,9 +390,17 @@ public class EditTemplateDialog extends StatusDialog {
 		return label;
 	}
 
+	private static Button createCheckbox(Composite parent, String name) {
+		Button button= new Button(parent, SWT.CHECK);
+		button.setText(name);
+		button.setLayoutData(new GridData());
+		
+		return button;
+	}
+	
 	private static Text createText(Composite parent) {
 		Text text= new Text(parent, SWT.BORDER);
-		text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		
+		text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		
 		return text;
 	}
@@ -381,12 +412,10 @@ public class EditTemplateDialog extends StatusDialog {
 		tools.setupJavaDocumentPartitioner(document, IJavaPartitions.JAVA_PARTITIONING);
 		IPreferenceStore store= JavaPlugin.getDefault().getCombinedPreferenceStore();
 		SourceViewer viewer= new JavaSourceViewer(parent, null, null, false, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL, store);
-		TemplateEditorSourceViewerConfiguration configuration= new TemplateEditorSourceViewerConfiguration(tools.getColorManager(), store, null, fTemplateProcessor);
+		CodeTemplateSourceViewerConfiguration configuration= new CodeTemplateSourceViewerConfiguration(tools.getColorManager(), store, null, fTemplateProcessor);
 		viewer.configure(configuration);
 		viewer.setEditable(true);
-		// XXX workaround for bug 63313 - disabling prefix until fixed.
-//		viewer.setDocument(document, prefix.length(), document.getLength() - prefix.length());
-		viewer.setDocument(document);
+		viewer.setDocument(document, prefix.length(), document.getLength() - prefix.length());
 		
 		Font font= JFaceResources.getFont(PreferenceConstants.EDITOR_TEXT_FONT);
 		viewer.getTextWidget().setFont(font);
@@ -396,7 +425,7 @@ public class EditTemplateDialog extends StatusDialog {
 		if (nLines < 5) {
 			nLines= 5;
 		} else if (nLines > 12) {
-			nLines= 12;	
+			nLines= 12;
 		}
 				
 		Control control= viewer.getControl();
@@ -412,77 +441,65 @@ public class EditTemplateDialog extends StatusDialog {
 			}
 		});
 
-		viewer.addSelectionChangedListener(new ISelectionChangedListener() {			
+		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
 			public void selectionChanged(SelectionChangedEvent event) {
 				updateSelectionDependentActions();
 			}
 		});
 
-		viewer.prependVerifyKeyListener(new VerifyKeyListener() {
-			public void verifyKey(VerifyEvent event) {
-				handleVerifyKeyPressed(event);
-			}
-		});
-		
 		return viewer;
 	}
 	
 	private String getPrefix() {
-		String prefix;
-		int idx= getIndex(fTemplate.getContextTypeId());
+		String id= getContextId();
+		int idx= getIndex(id);
 		if (idx != -1)
-			prefix= fContextTypes[idx][2];
+			return fContextTypes[idx][2];
 		else
-			prefix= ""; //$NON-NLS-1$
-
-		return prefix;
-	}
-
-	private void handleVerifyKeyPressed(VerifyEvent event) {
-		if (!event.doit)
-			return;
-
-		if (event.stateMask != SWT.MOD1)
-			return;
-			
-		switch (event.character) {
-			case ' ':
-				fPatternEditor.doOperation(ISourceViewer.CONTENTASSIST_PROPOSALS);
-				event.doit= false;
-				break;
-
-			// CTRL-Z
-			case 'z' - 'a' + 1:
-				fPatternEditor.doOperation(ITextOperationTarget.UNDO);
-				event.doit= false;
-				break;				
-		}
+			return ""; //$NON-NLS-1$
 	}
 
 	private void initializeActions() {
-		TextViewerAction action= new TextViewerAction(fPatternEditor, SourceViewer.UNDO);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.undo")); //$NON-NLS-1$
-		fGlobalActions.put(ITextEditorActionConstants.UNDO, action);
+		final ArrayList handlerActivations= new ArrayList(3);
+		final IHandlerService handlerService= (IHandlerService) PlatformUI.getWorkbench().getAdapter(IHandlerService.class);
+		getShell().addDisposeListener(new DisposeListener() {
+			public void widgetDisposed(DisposeEvent e) {
+				handlerService.deactivateHandlers(handlerActivations);
+			}
+		});
+		
+		Expression expression= new ActiveShellExpression(fPatternEditor.getControl().getShell());
 
-		action= new TextViewerAction(fPatternEditor, SourceViewer.CUT);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.cut")); //$NON-NLS-1$
+		TextViewerAction action= new TextViewerAction(fPatternEditor, ITextOperationTarget.UNDO);
+		action.setText(PreferencesMessages.EditTemplateDialog_undo);
+		fGlobalActions.put(ITextEditorActionConstants.UNDO, action);
+		handlerActivations.add(handlerService.activateHandler(IWorkbenchActionDefinitionIds.UNDO, new ActionHandler(action), expression));
+
+		action= new TextViewerAction(fPatternEditor, ITextOperationTarget.REDO);
+		action.setText(PreferencesMessages.EditTemplateDialog_redo);
+		fGlobalActions.put(ITextEditorActionConstants.REDO, action);
+		handlerActivations.add(handlerService.activateHandler(IWorkbenchActionDefinitionIds.REDO, new ActionHandler(action), expression));
+
+		action= new TextViewerAction(fPatternEditor, ITextOperationTarget.CUT);
+		action.setText(PreferencesMessages.EditTemplateDialog_cut);
 		fGlobalActions.put(ITextEditorActionConstants.CUT, action);
 
-		action= new TextViewerAction(fPatternEditor, SourceViewer.COPY);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.copy")); //$NON-NLS-1$
+		action= new TextViewerAction(fPatternEditor, ITextOperationTarget.COPY);
+		action.setText(PreferencesMessages.EditTemplateDialog_copy);
 		fGlobalActions.put(ITextEditorActionConstants.COPY, action);
 
-		action= new TextViewerAction(fPatternEditor, SourceViewer.PASTE);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.paste")); //$NON-NLS-1$
+		action= new TextViewerAction(fPatternEditor, ITextOperationTarget.PASTE);
+		action.setText(PreferencesMessages.EditTemplateDialog_paste);
 		fGlobalActions.put(ITextEditorActionConstants.PASTE, action);
 
-		action= new TextViewerAction(fPatternEditor, SourceViewer.SELECT_ALL);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.select.all")); //$NON-NLS-1$
+		action= new TextViewerAction(fPatternEditor, ITextOperationTarget.SELECT_ALL);
+		action.setText(PreferencesMessages.EditTemplateDialog_select_all);
 		fGlobalActions.put(ITextEditorActionConstants.SELECT_ALL, action);
 
-		action= new TextViewerAction(fPatternEditor, SourceViewer.CONTENTASSIST_PROPOSALS);
-		action.setText(PreferencesMessages.getString("EditTemplateDialog.content.assist")); //$NON-NLS-1$
+		action= new TextViewerAction(fPatternEditor, ISourceViewer.CONTENTASSIST_PROPOSALS);
+		action.setText(PreferencesMessages.EditTemplateDialog_content_assist);
 		fGlobalActions.put("ContentAssistProposal", action); //$NON-NLS-1$
+		handlerActivations.add(handlerService.activateHandler(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS, new ActionHandler(action), expression));
 
 		fSelectionActions.add(ITextEditorActionConstants.CUT);
 		fSelectionActions.add(ITextEditorActionConstants.COPY);
@@ -497,7 +514,7 @@ public class EditTemplateDialog extends StatusDialog {
 			}
 		});
 
-		StyledText text= fPatternEditor.getTextWidget();		
+		StyledText text= fPatternEditor.getTextWidget();
 		Menu menu= manager.createContextMenu(text);
 		text.setMenu(menu);
 	}
@@ -505,8 +522,9 @@ public class EditTemplateDialog extends StatusDialog {
 	private void fillContextMenu(IMenuManager menu) {
 		menu.add(new GroupMarker(ITextEditorActionConstants.GROUP_UNDO));
 		menu.appendToGroup(ITextEditorActionConstants.GROUP_UNDO, (IAction) fGlobalActions.get(ITextEditorActionConstants.UNDO));
+		menu.appendToGroup(ITextEditorActionConstants.GROUP_UNDO, (IAction) fGlobalActions.get(ITextEditorActionConstants.REDO));
 		
-		menu.add(new Separator(ITextEditorActionConstants.GROUP_EDIT));		
+		menu.add(new Separator(ITextEditorActionConstants.GROUP_EDIT));
 		menu.appendToGroup(ITextEditorActionConstants.GROUP_EDIT, (IAction) fGlobalActions.get(ITextEditorActionConstants.CUT));
 		menu.appendToGroup(ITextEditorActionConstants.GROUP_EDIT, (IAction) fGlobalActions.get(ITextEditorActionConstants.COPY));
 		menu.appendToGroup(ITextEditorActionConstants.GROUP_EDIT, (IAction) fGlobalActions.get(ITextEditorActionConstants.PASTE));
@@ -515,17 +533,12 @@ public class EditTemplateDialog extends StatusDialog {
 		menu.add(new Separator(IContextMenuConstants.GROUP_GENERATE));
 		menu.appendToGroup(IContextMenuConstants.GROUP_GENERATE, (IAction) fGlobalActions.get("ContentAssistProposal")); //$NON-NLS-1$
 	}
+	
 
 	protected void updateSelectionDependentActions() {
 		Iterator iterator= fSelectionActions.iterator();
 		while (iterator.hasNext())
-			updateAction((String)iterator.next());		
-	}
-
-	protected void updateUndoAction() {
-		IAction action= (IAction) fGlobalActions.get(ITextEditorActionConstants.UNDO);
-		if (action instanceof IUpdate)
-			((IUpdate) action).update();
+			updateAction((String)iterator.next());
 	}
 
 	protected void updateAction(String actionId) {
@@ -541,38 +554,83 @@ public class EditTemplateDialog extends StatusDialog {
 		
 		for (int i= 0; i < fContextTypes.length; i++) {
 			if (contextid.equals(fContextTypes[i][0])) {
-				return i;	
+				return i;
 			}
 		}
 		return -1;
 	}
 	
 	protected void okPressed() {
+		String name= fNameText == null ? fTemplate.getName() : fNameText.getText();
+		boolean isAutoInsertable= fAutoInsertCheckbox != null && fAutoInsertCheckbox.getSelection();
+		fTemplate= new Template(name, fDescriptionText.getText(), getContextId(), getPattern(), isAutoInsertable);
 		super.okPressed();
 	}
 	
-	private void updateButtons() {		
-		StatusInfo status;
-
-		boolean valid= fNameText == null || fNameText.getText().trim().length() != 0;
-		if (!valid) {
-			status = new StatusInfo();
-			if (!fSuppressError) {
-				status.setError(PreferencesMessages.getString("EditTemplateDialog.error.noname")); //$NON-NLS-1$
-			}
- 		} else {
- 			status= fValidationStatus; 
- 		}
+	private void updateStatusAndButtons() {
+		StatusInfo status= fValidationStatus;
+		boolean isEmpty= fNameText != null && fNameText.getText().length() == 0;
+		if (!fSuppressError && isEmpty) {
+			status= new StatusInfo();
+			status.setError(PreferencesMessages.EditTemplateDialog_error_noname);
+		} else if (fNameText != null && !isValidTemplateName(fNameText.getText())) {
+			status= new StatusInfo();
+			status.setError(PreferencesMessages.EditTemplateDialog_error_invalidName);
+		}
 		updateStatus(status);
 	}
 
+	/**
+	 * Checks whether the given string is a valid
+	 * template name.
+	 * 
+	 * @param name the string to test
+	 * @return <code>true</code> if the name is valid
+	 * @since 3.3.1
+	 */
+	private boolean isValidTemplateName(String name) {
+		return name.length() == 0 || name.trim().length() != 0;
+	}
+
 	/*
 	 * @see org.eclipse.jface.window.Window#configureShell(Shell)
 	 */
 	protected void configureShell(Shell newShell) {
 		super.configureShell(newShell);
-		WorkbenchHelp.setHelp(newShell, IJavaHelpContextIds.EDIT_TEMPLATE_DIALOG);
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(newShell, IJavaHelpContextIds.EDIT_TEMPLATE_DIALOG);
 	}
 
+	/**
+	 * Returns the created template.
+	 * 
+	 * @return the created template
+	 * @since 3.1
+	 */
+	public Template getTemplate() {
+		return fTemplate;
+	}
+	
+	private String getPattern() {
+		IDocument doc= fPatternEditor.getDocument();
+		IRegion visible= fPatternEditor.getVisibleRegion();
+		try {
+			return doc.get(visible.getOffset(), doc.getLength() - visible.getOffset());
+		} catch (BadLocationException e) {
+			return ""; //$NON-NLS-1$
+		}
+	}
+	
+	/*
+	 * @see org.eclipse.jface.dialogs.Dialog#getDialogBoundsSettings()
+	 * @since 3.2
+	 */
+	protected IDialogSettings getDialogBoundsSettings() {
+		String sectionName= getClass().getName() + "_dialogBounds"; //$NON-NLS-1$
+		IDialogSettings settings= JavaPlugin.getDefault().getDialogSettings();
+		IDialogSettings section= settings.getSection(sectionName);
+		if (section == null)
+			section= settings.addNewSection(sectionName);
+		return section;
+	}
 
 }
